import omit from 'lodash/omit'

export function routes(options = {}, context = {}) {
  const { project = this } = context

  return [
    '*package.json',
  ]
}

export function collectionMethods(options = {}, context = {}) {
  const { project = this } = context

  return {}
}

export function instanceMethods(options = {}, context = {}) {
  const { project = this } = context

  const { spawn, fork, exec } = require('child-process-promise')

  return {
    spawn(command, options = {}) {
      const { out, err } = options 

      if (options.capture !== false) {
        options.capture = ['stdout', 'stderr']  
      }

      const promise = spawn(process.argv[1], command.split(' '), {
        cwd: project.resolve(this.relativeDirname),
        shell: true,
        ...omit(options, 'out', 'err'),
        env: {
          FORCE_COLOR: true,
          ...process.env,
          ...options.env || {},
        },
      })

      if (typeof out === 'function') {
        out.call(this, promise.childProcess.stdout, options, context)
      }

      if (typeof err === 'function') {
        err.call(this, promise.childProcess.stdout, options, context)
      }

      return promise
    },
    exec(command, options = {}) {
      return exec(`${process.argv[1]} ${command}`, {
        cwd: this.relativeDirname,
        ...options,
        env: {
          FORCE_COLOR: true,
          ...process.env,
          ...options.env || {},
        },       
      })
    },    
    fork(script, options = {}) {
      return fork(project.resolve(this.relativeDirname, script), {
        cwd: this.relativeDirname,
        ...options,
        env: {
          FORCE_COLOR: true,
          ...process.env,
          ...options.env || {},
        },       
      })
    }
  }
}